function rpd=mkraydepth(p,v,r,h,rp);
% mkraydepth.........compute ray penetration depth IN SPHERICAL EARTH
%
% call: rpd=mkraydepth(p,v,r,h,rp);
%
%           p: ray parameter [s/rad]
%           v: velocities [km/s] within planet at radii given in R
%           r: radii at which velocities are defined [km]
%              R(1) is the largest radius (=the smallest depth)
%              R(1)>=R(2)>=R(3)...
%           h: radius at which the focus is
%           rp: planet radius [km]
%
% result: rpd: ray penetration depth: smallest radius (=greatest depth)
%              reached by ray with ray parameter P. At this radius, the
%              ray bends back towards the surface.
%              BEWARE: RPD may contain more than one solution. Use the
%                      first one that is below your Hypocenter!
%                      (this is not a bug!)
%                      DO NOT ASSUME A SPECIFIC ORDER. USE MAX() TO FIND
%                      THE SHALLOWEST ONE!
%              NaN if ray does not bend back.
%
% the velocity is assumed to be piecewise linear: v(r)=v0-g*r
% Piecewise constant models won't have ray vortices inside layers.
%
% possible vortex depths are independent of the takeoff angle of the ray.
% You have to decide if vortex depths are important at all! (for a ray
% which goes upward from the source, vortices below the source are not
% relevant!)
%
% input parameters are in spherical coordinates. The Earth flattening
% transformation is applied internally, but to v only. This is why the
% planet's radius is needed.
%
% Martin Knapmeyer, 22.04.2002, 12.11.2003-17.11.2003, 24.03.2004

%%% init result
res=NaN;

%%% v and z have to be row vectors
v=v(:)';
r=r(:)';




%%% compute velocity gradients within all layers
%%% to suppres "divide by zero"-warnings, gradient at 1st-order 
%%% discontinuities is set to inf by hand
%%% g(1) is the gradient from v(1) to v(2) at depths z(1) to z(2)
anz=length(v); % so many parameters
r1=r(1:anz-1); % upper edges of layers
r2=r(2:anz);   % lower edges of layers
v1=v(1:anz-1); % velocities at upper edges
v2=v(2:anz);   % velocities at lower edges
warning off    % to suppress ugly "divide by zero" warnings resulting from discontinuities
g=(v2-v1)./(r1-r2); % velocity gradients
warning on     % allow warnings later, because they're unexpected

%%% at depths where v(z) has a kink, g will be 0/0=NaN. It makes physical
%%% sense to replace these NaN gradients by a zero gradient.
nangradients=find(isnan(g));
g(nangradients)=zeros(size(nangradients));



%%% compute radius at which vertex is (where ray bends back)
%%% four steps:
%%% 1. find possible vertex radii for all layers given in v and r
%%%    These radii are computed from snellius and the v(z) law.
%%%    The result is a radius (or vertex depth) which assumes that
%%%    the same linear velocity law is valid not only within one
%%%    layer but within the entire planet.
%%%    This is where the velocity interpolation occurs.
%%% 1b. Identify discontinuities which would totally relfect the current ray
%%% 2. find the layer, for which the vertex is within the layer for
%%%    which the law is really valid.
%%%    There may be more than one layer in which this is the case.
%%% 3. Remove solutions forbidden by LVZs (MK12112003)

%%% step 1
rs=p*(v1+g.*r1)./(1+p*g);

%%% step 1b
%%% we have to test all discontinuities for
%%% fulfilling the total reflection condition for the current ray parameter
%%% first identify discontinuities: g is inf there.
discons=find(isinf(g));
%%% now test each of them and add those that would totally reflect the
%%% ray to CANDIDATES.
reflections=[];
for disconcnt=1:length(discons)
    if v2(discons(disconcnt))~=0
       %%% v~=0: test for criticval angle condition   
       if p>r1(discons(disconcnt))/v2(discons(disconcnt))
          %%% this condision means: discontinuity could totally reflect ray
          rs(discons(disconcnt))=r1(discons(disconcnt));
          reflections=[reflections discons(disconcnt)];
       end; % if p>...
    else
       %%% v==0: velocity is zero, no wave can enter: all wavesd reflected!
       rs(discons(disconcnt))=r1(discons(disconcnt));
       reflections=[reflections discons(disconcnt)];
    end; % if v2(discons(disconcnt))~=0
end; % for disconcnt

%%% step 2
refractions=find((rs>=r2)&(rs<=r1));
candidates=[refractions];
if isempty(candidates)
   %%% no solution
   res=NaN;
   %disp('MKRAYDEPTH: no vertex candidates (reason 1)');
else
   %%% remove solutions within any LVZ
   
   %%% consider only the candidate radii
   rcand=rs(candidates);
   
   %%% Flat Earth Transform for velocity
   %%% we don't use MKSFER2FLAT here because only v is to be transformed
   rzero=find(r==0);
   r(rzero)=r(rzero)*0-1; % to avoid divide-by-zero warnings
   vflat=rp*v./r; % rp is planet radius
   vflat(rzero)=abs(vflat(rzero))*inf;
   r(rzero)=r(rzero)*0;
   %%% Flat Earth transform done
   
   %%% which of the candidate vertices are impossible due to LVZ?
   %%% determine velocity at focal depth
   vfocus=mkinterp(r,vflat,h);
   %%% determine flat velocities at candidate depths
   vcand=mkinterp(r,vflat,rcand);
   %%% loop through all candidates and test them
   %%% impossible focus depths are set to NaN
   for candnr=1:length(rcand)
       if vcand(candnr)<=vfocus
          rs(candnr)=NaN;
          %disp('MKRAYDEPTH: vertex removed (reason 2)');
       else
          %%% determine all velocities between focus radius and candidate radius
          indies=find((r<=h)&(r>=rcand(candnr)));
          if vcand(candnr)<=max(vflat(indies))
             rs(candnr)=NaN;
             %disp('MKRAYDEPTH: vertex removed (reason 3)');
          end; % if 
       end; % if vcand<=vfocus
   end; % for candnr
   
   remain=find(~isnan(rcand)); % dummy line, removing nothing
   
   %%% construct list of remaining candidate solutions
   if isempty(remain)
      %%% no solution, again
      res=NaN;
   else
      %%% the winner is...
      %%% BEWARE: RS may contain more than one solution. Use only the
      %%% first one that is below your Hypocenter!
      res=rcand(remain);
   end; % if isempty(remain)
end; 

%%% return result
rpd=res;




