function [rayhandles,texthandles]=mkraydiagram(p1,p2,p3,p4,p5,p6,p7);
% mkraydiagram.......plots nice diagram of ray paths
%
% call: [rayhandles,texthandles]=mkraydiagram(delta,z,type,rp);
%       mkraydiagram(delta,z,type,rp,'addlabel');
%       mkraydiagram(delta,z,type,dz,dname,rp);
%       mkraydiagram(delta,z,type,dz,dname,rp,'addlabel');
%       mkraydiagram(dz,dname,rp);
%       mkraydiagram(dz,dname,rp,'addlabel');
%
%       delta: vector of epicentral distances [deg]
%              describing the "horizontal" component
%              of the ray geometry
%       z: vector of depths [km]
%          descrbing the "vertical" component of ray geometry
%       type: string conntaining a 'P' or 'S' for each ray segment, defining the
%             wave type (as returned by MKX4P)
%       rp: planetary radius [km]
%       dz: list of discontinuity depths
%       dname: list of discontinuity names
%
% result: rayhandles: vector of handles to all generated line objects (rays only)
%         texthandles: vecxtor of handles to all generated text objects (annotations)
%
% This routine creates nice plots of ray paths.
% Three modi are available:
% 1) mkraydiagram(delta,z,type,rp);
%    mkraydiagram(delta,z,type,rp,'addlabel');
%    plot ray into existing diagram - simply plots the ray to the screen,
%    independent of what ios on the screen (YGWYD / you get what you deserve)
% 2) mkraydiagram(dz,dname,rp);
%    mkraydiagram(dz,dname,rp,'addlabel');
%    initialize a new diagram: clears screen & creates polar coordinate system 
%    with angular scale, discontinuities etc.
% 3) mkraydiagram(delta,z,type,dz,dname,rp);
%    mkraydiagram(delta,z,type,dz,dname,rp,'addlabel');
%    initialize new diagram and plot rayn into it
%
% Specifying 'addlabel' as last parameter causes MKRAYDIAGRAM to add angular
% labels around the diameter of the plot.
% In mode 2, labels are set around the whole diameter. In modes 1 and 3, labels
% are set only at points where rays arrive.
%
% rays are plottet piecewise linear between coordinates given in DELTA and Z.
% If you wish to have a better approximiation to the real ray shape, you
% have to use a velocity model with thinner layers.
%
% Martin Knapmeyer
%
% BUGS: - when plotting phases like PS, the last ray segment gets lost, although computed
%         correctly.
%       - when plotting Phases like SKS, the ray color at the CMB is not correct, but
%         timing and ray geometry is correct.


%%% useful constants
radian=pi/180;
fontsize=10;

%%% initialize some arrays
th=[]; % to collect text handles

%%% understand input
nin=nargin;
switch nin
   case {3} %mkraydiagram(dz,dname,rp);
      mode=2;
      dz=p1;
      dname=p2;
      rp=p3;
      type=NaN;
      addlabel=0;
      label360=0;
   case {4} %mkraydiagram(delta,z,type,rp); or mkraydiagram(dz,dname,rp,'addlabel');
      if ischar(p4)
         mode=2;
         dz=p1;
         dname=p2;
         rp=p3;
         addlabel=0;
         label360=1;
         type=NaN;
      else
         mode=1;
         delta=p1;
         z=p2;
         type=lower(p3);
         rp=p4;
         addlabel=0;
         label360=0;
      end; % if ischar
   case {5} %mkraydiagram(delta,z,type,rp,'addlabel');
         mode=1;
         delta=p1;
         z=p2;
         type=lower(p3);
         rp=p4;
         addlabel=1;
         label360=1;
   case {6} %mkraydiagram(delta,z,type,dz,dname,rp);
      mode=3;
      delta=p1;
      z=p2;
      type=lower(p3);
      dz=p4;
      dname=p5;
      rp=p6;
      addlabel=0;
      label360=0
   case {7} %mkraydiagram(delta,z,type,dz,dname,rp,'addlabel');
      mode=3;
      delta=p1;
      z=p2;
      type=lower(p3);
      dz=p4;
      dname=p5;
      rp=p6;
      addlabel=1;
      label360=1;
   otherwise
      error('MKRAYDIAGRAM: use 3, 4, 5, 6, or 7 input parameters!');
 end; % switch nin
 
 
%%% tickmark definitions 
framewidth=0.05; % relative width of frame for ticks
steps=[1 5 10 90]; % tickmark intervals
ticklen=[0.3 0.75 1 2]*framewidth; % tickmark lenghts: tickmarks of interval spteps(1) will habe ticklen(1)
dolabel=[0 0 1 0]; % whether or not to write labels (only in case label360==1)

   
   
 %%% init new coordinate frame
 if (mode==2)|(mode==3)
    cla;
    n=720; % circle is an (n-1)-sided polygon
    phi0=-(2*pi)/n;
    phi360=2*pi;
    % framing circles
    mkcircle(0,0,rp,phi0,phi360,n,'k');
    hold on;
    mkcircle(0,0,rp*(1+framewidth),phi0,phi360,n,'k');
    hold off;
    % tickmarks as angular scale within framing circles
    anz=length(steps); % so many types of labels to draw
    hold on
    for i=1:anz
       for angle=0:steps(i):360;
          % x plot coordinates
          x=rp*cos((180-angle)*radian)*[1 1+ticklen(i)];
          % y plot coordinates
          y=rp*sin((180-angle)*radian)*[1 1+ticklen(i)];
          % plot
          plot(x,y,'k');
          % add labels
          if (dolabel(i)==1)&(angle<360)&(label360==1)
             if angle<100
                strangle=[' ' int2str(mod(fix(180-angle),360))];
             else
                strangle=int2str(mod(fix(180-angle),360));
             end; % if angle<100
             h=text((rp*(1+ticklen(anz)))*cos(angle*radian),...
                   (rp*(1+ticklen(anz)))*sin(angle*radian),...
                   strangle);
             set(h,'Rotation',angle);
             set(h,'FontSize',fontsize);
             th=[th h];
          end; % if dolabel
       end; % for j
    end; % for i
    % discontinuities
    anz=length(dz); % so many discontinuities
    for i=1:anz
       mkcircle(0,0,rp-dz(i),phi0,phi360,n,'k');
    end; % for i
    % justify axes
    hold off
    axis tight
    %axis equal
    axis square
    %axis auto
    axis off
 end; % if mode==2 | mode ==3
 
 
 %%% plot ray
 anz=length(type); % so many ray segments to plot
 handles=zeros(1,anz);
 if ((mode==1)|(mode==3))&(anz>1)
    hold on
    for indy=1:anz;
       switch type(indy)
          case{'p','P'}
             style='b';
          case{'s','S'}
             style='r';
          otherwise
             error(['MKRAYDIAGRAM: type ' type ' unknown.']);
       end; % switch type
       x=[(rp-z(indy)).*cos(mod(180-delta(indy),360).*radian)...
          (rp-z(indy+1)).*cos(mod(180-delta(indy+1),360).*radian)];
       y=[(rp-z(indy)).*sin(mod(180-delta(indy),360).*radian)...
             (rp-z(indy+1)).*sin(mod(180-delta(indy+1),360).*radian)] ;
       %%% this plots the ray segment number INDY
       handles(indy)=plot(x,y,style);
       set(handles(indy),'tag',['ray ' type(indy)]);
    end; % for indy
    %%% and this adds annotation.
    if addlabel==1
       angle=mod(180-delta(length(delta)),360);
       if ~isnan(angle)
          roundedangle=round(angle*10)/10;
          if angle<100
             strangle=[' ' num2str(180-roundedangle)];
          else
             strangle=num2str(180-roundedangle);
          end; % if angle<100
          h=text((rp*(1+max(ticklen)))*cos(angle*radian),...
                (rp*(1+max(ticklen)))*sin(angle*radian),...
                strangle);
          set(h,'Rotation',angle);
          set(h,'FontSize',fontsize);
          th=[th h];
       end; % if ~isnan(angle)
    end; % if addlabel==true
    hold off
 end; % if mode==1 | mode==3
 
 %%% modify axes: labeling must not overwrite axes title
 axis(mkgetaxes(gca));
 
 %return only non-zeros handles, since 0 is the root object...
 rayhandles=handles(find(handles~=0));
 texthandles=th;
 
   