function angles=mksmarttakeoff(phase,model,h,dangle);
% mksmarttakeoff.......compute phase dependently optimized takeoff angles
%
% call: angles=mksmarttakeoff(phase,model,h,dangle);
%
%       phase: a seismological phase name
%       model: velocity model structure, as returned by MKREADND
%       h: focal depth [km]
%       dangle: delta angle, take off angle resolution [deg]
%
%
% result: angles: optimized list of takeoff angles [deg]
%                 If PHASE is an unknown phase name, the full
%                 range dangle:dangle:90 is returned.
%                 Empty in case of problems.
%
% depending on the phase, it is not necessary to scan the whole
% 0...90deg range of takeoff angles when searching for ray paths
% or travel times. This routine produces an optimized angle list.
%
% Martin Knapmeyer, 23.09.2003

%%% init result
angles=[];

%%% useful, as usually
radian=pi/180;
angleeps=1e-3;

%%% velocities at focal depth
focus=mkinterpmodel(model,h,'simple');
%%% determine ray parameter needed to touch CMB
%%% or smallest ray parameter for which take off angle is not complex
if ~isnan(model.cmb)
   pcmb=mkprad2deg(mkraydepthinv(model.rp-model.cmb,model.vp,model.rp-model.z));
   scmb=mkprad2deg(mkraydepthinv(model.rp-model.cmb,model.vs,model.rp-model.z));
   %%% ray parameter beyond complex-limit?
   if pcmb>(model.rp-h)/focus.vp*pi/180
      pcmb=(model.rp-h)/focus.vp*pi/180-eps;
   end; % if pcmb>(model.rp-h)/focus.vp*pi/180
   if scmb>(model.rp-h)/focus.vs*pi/180
      scmb=(model.rp-h)/focus.vs*pi/180-eps;
   end; % if pcmb>(model.rp-h)/focus.vp*pi/180
   %%% transform ray parameters into take off angles
   pcmbangle=asin(((pcmb/radian)*focus.vp)/(model.rp-h))/radian;
   scmbangle=asin(((scmb/radian)*focus.vs)/(model.rp-h))/radian;
   %%% due to rounfoff, pcmbangle and scmbangle might still have
   %%% a small imaginary component
   if (imag(pcmbangle)<angleeps)&(imag(scmbangle)<angleeps)
      pcmbangle=real(pcmbangle);
      scmbangle=real(scmbangle);
   else
      error('MKSMARTTAKEOFF: complex take off generated for CMB touch');
   end; % if (imag(pcmbangle)<angleeps)&(imag(scmbangle)<angleeps)
else
   pcmb=NaN;
   scmb=NaN;
   pcmbangle=NaN;
   scmbangle=NaN;
end;  % if ~isnan(model.cmb)
if ~isnan(model.icb)
   picb=mkprad2deg(mkraydepthinv(model.rp-model.icb,model.vp,model.rp-model.z));
   sicb=picb; % because its a P wave at the ICB!
   %%% ray parameter beyond complex-limit?
   if picb>(model.rp-h)/focus.vp*pi/180
      picb=(model.rp-h)/focus.vp*pi/180-eps;
   end; % if pcmb>(model.rp-h)/focus.vp*pi/180
   if sicb>(model.rp-h)/focus.vp*pi/180
      sicb=(model.rp-h)/focus.vp*pi/180-eps;
   end; % if pcmb>(model.rp-h)/focus.vp*pi/180
   %%% transform ray parameters into take off angles
   picbangle=asin(((picb/radian)*focus.vp)/(model.rp-h))/radian;
   sicbangle=asin(((sicb/radian)*focus.vs)/(model.rp-h))/radian;
   %%% due to rounfoff, picbangle and sicbangle might still have
   %%% a small imaginary component
   if (imag(picbangle)<angleeps)&(imag(sicbangle)<angleeps)
      picbangle=real(picbangle);
      sicbangle=real(sicbangle);
   else
      if ~isempty(strfind(phase,'K'))
         %%% since touching the ICB is relevant only for core phases, we
         %%% may ignore problems in non-core-phase case.
         error('MKSMARTTAKEOFF: complex take off generated for ICB touch');
      end; % if ~isempty(strfind(phase,'K'))
   end; % if (imag(pcmbangle)<angleeps)&(imag(scmbangle)<angleeps)
else
   picb=NaN;
   sicb=NaN;
   picbangle=dangle;
   sicbangle=dangle;
end; % if ~isnan(model.icb)


%%% depth phase handling, part 1
isdepthphase=0;
if strcmp(phase(1),'s')| strcmp(phase(1),'p')
   phase=phase(2:end);
   isdepthphase=1;
end; % if strcmp(phase(1),'s')| strcmp(phase(1),'p')


%%% define phase dependent angle range
switch phase
   case {'P','PP','PPP'}        
        minangle=pcmbangle;
        if isnan(minangle)
           minangle=dangle;
        end; % if isnan(minangle)
        if h>0
           maxangle=180;
        else
           maxangle=90;
        end; % if h>0
   case {'PcP','PcPPcP'}        
        minangle=dangle;
        maxangle=pcmbangle;
   case {'PKP','PKKP','PKPPKP'}        
        minangle=dangle;
        maxangle=pcmbangle;
   case {'PKiKP','PKIKP','PKIKPPKIKP'}        
        minangle=dangle;
        maxangle=picbangle;
   case {'S','SS','SSS'}        
        minangle=scmbangle;
        if isnan(minangle)
           minangle=dangle;
        end; % if isnan(minangle)
        if h>0
           maxangle=180;
        else
           maxangle=90;
        end; % if h>0
   case {'ScS','ScSScS'}        
        minangle=dangle;
        maxangle=scmbangle;
   case {'SKS','SKKS','SKSSKS'}        
        minangle=dangle;
        maxangle=scmbangle;
   case {'SKiKS','SKIKS'}        
        minangle=dangle;
        maxangle=sicbangle;
   case {'PS','SP'}
        minangle=min([scmbangle pcmbangle]);
        if isnan(minangle)
           minangle=dangle;
        end; % if isnan(minangle)
        if h>0
           maxangle=180;
        else
           maxangle=90;
        end; % if h>0
   case {'PcS','ScP'}
        minangle=min([sicbangle picbangle]);
        maxangle=min([scmbangle pcmbangle]);
   otherwise
       disp(['MKSMARTTAKEOFF: unknown phase ' phase ', using default.']);
       minangle=dangle;
       if h>0
           maxangle=180;
        else
           maxangle=90;
        end; % if h>0
end; % switch phase

%%% depth phase handling, part 2
if isdepthphase
   if minangle<90
      minangle=90;
   end; % if minangle
end; % if isdepthphase


%%% construct angle list from range and delta
angles=minangle:dangle:maxangle;

if isnan(angles)
   angles=[];
else
   if angles(end)~=maxangle
      angles=[angles maxangle];
   end; % if angles(end)
end; % if isnan(angles)