function yneu=mkinterp(xalt,yalt,xneu);
% mkinterp..............one dimensional interpolation of nonmonotonic functions
%
% call: yneu=mkinterp(xalt,yalt,xneu);
%
%            xalt: x values at which function f(x) is given
%            yalt: function values yalt=f(xalt)
%            xneu: x values at which function f(x) is requested
%                  all values of XALT have to be between the smallest XALT and
%                  the largest XALT (or equal to these) - the routine does not
%                  do EXTRApolation, but INTERpolation.
%
% result: yneu:  yneu=f(xneu)
%                three cases will be distinguished:
%                1) xneu is identical to one of the xalt values
%                   yneu will be the corresponding yalt
%                2) xneu is no identical to one of the xalt values
%                   yneu will be determined by linear interpolation
%                3) xneu is identical to several xalt values
%                   yneu will be determined as mean of f(xalt1), f(xalt2),...
%
%
% MatLab's interp1-function requires x to be monotonic for purely technical
% reasons: vectorization as done there is impossible for nonmonotonic x series.
% That is ridiculous.
%
% This routine is not vectorized and therefore slower :-( 
% This routine does not make any assumptions on x and y. f(x) does not even
% have to be unique!
% Only linear interpolation is possible.
%
% Martin Knapmeyer 28.01.2000


xlen=length(xneu);
minxalt=min(xalt);
maxxalt=max(xalt);
yneu=zeros(size(xneu));

for i=1:xlen
    if (xneu(i)>=minxalt)&(xneu(i)<=maxxalt)
       indies=find(xalt==xneu(i));
       inlen=length(indies);
       switch inlen
          case 0,
               % i-th xneu is truely between the given xalt values
               % search the limits to interpolate between
               xdiff=xalt-xneu(i);
               [xdiffsort,sortindy]=sort(xdiff);
               xalt=xalt(sortindy); % sort x by distance to xneu
               yalt=yalt(sortindy); % sort y by distance of x to xneu
               indy=find(diff(sign(xdiffsort))==2);
               x0=xalt(indy);   % largest x xmaller than xneu(i)
               x1=xalt(indy+1); % smallest x larger than xneu(i)
               y0=yalt(indy);   % y1=f(x1)
               y1=yalt(indy+1); % y2=f(x2)
               % interpolate
               h=x1-x0;
               delta=y1-y0;
               yneu(i)=y0+delta*(xneu(i)-x0)/h;
          case 1,
               % i-th xneu is identical to one of the xalt values
               yneu(i)=yalt(indies);
          otherwise,
               % i-th xneu is identical to several xalt values -> compute mean
              yneu(i)=mean(yalt(indies));
       end; % switch
    else
       error(['MKINTERP: XNEU(' int2str(i)...
             ') is outside x range defined by XALT!']);
    end; % if (xneu(i)>=minxalt)&(xneu(i)=<maxxalt)
end; % for i
